/***************************************************************************
                          lconfig.h  -  description
                             -------------------
    begin                : Fri Jan 17 2003
    copyright            : (C) 2003 by Sheldon Lee Wen
    email                : leewsb@hotmail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef LCONFIG_H
#define LCONFIG_H

#if __GNUC__ >= 3
   #include <fstream>
#else
   #include <fstream.h>
#endif

#include <config.h>

#include <map>
#include <vector>
#include <iostream>
#include <lineak/definitions.h>
#include <lineak/configdirectives.h>
#include <lineak/lconfigdata.h>
#include <lineak/lkbd.h>
#include <lineak/lkey.h>
#include <lineak/lcommand.h>

/**Lineak user config file. The LConfig class is a subclass of the LConfigData class.
  *An LConfig object contains all of the configuration file directives information in a
  *ConfigDirectives object. (Directives are all of the global configuration options that
  *are not keys and command. e.g. CdromDevice.) It also contains commands for each key
  *in the form of a map of LCommand objects, that is keyed on the key name (string).
  *@author Sheldon Lee Wen
  */
class LConfig : public LConfigData {
public:

	/**
	  There are 4 ways to instantiate an LConfig object.
		1) Default constructor, will be an empty object.
		2) Copy constructor.
		3) Constructor taking a ConfigDirectives object. This is the recommended way.
		4) Constructor created with minimal values of the keyboard type, cdrom device,
	   and mixer device. This is not the recommended way to create this object,
	   and is deprecated.
        */
	LConfig();
	LConfig(const LConfig &rhs);
	LConfig(ConfigDirectives &directives);
	// NOTSUPPORTED ANYMORE LConfig(string ikbdtype, string icdrom, string imixer);
	LConfig & operator=(const LConfig &rhs);
	virtual ~LConfig();
	
	/**
	The setValue and getValue/getIntValue methods are exactly the same as the
	ConfigDirectives methods addValue and getValue/getIntValue. In fact, they are
	just a wrapper around them for the private ConfigDirectives variable.
	The [] operator just returns a reference a the ConfigDirectives string value
	*/
  	void setValue(string key, string value);
  	void setValue(string key, int value);
  	string getValue(string key) const;
  	int getIntValue(string key) const;
  	string& operator[](string key) { return directives[key]; }


  	/** Return a list of keynames that we have LCommands for. */
  	const vector<string>& getCommandKeyNames();
 	/** Return a map of commands indexed by keycodes. */
  	map<const int,  LCommand> getCommandIndex(const map< string, LKey>& skeys) ;
 	/** Return a map of commands indexed by keycodes. */
  	map<const int,  LCommand> getCommandIndex(const LKbd& keyboard) ;
  	/** Return a command string given a key code */
	LCommand& getCommand(string key);
	/** Set the command for a key */
	void setCommand(string key, LCommand command);
	/** Add a key and command mapping. */
	void addKey(string key, LCommand command, bool respect_toggle = false);
	inline string getFilename() const { return directives.getValue("conffilename"); }

	/** The setFilename method is used to set the name of the file that the LConfig
	    was parsed from. It is necessary that the filename and keyboard type be set
	    for the isEmpty() method to work. */
	void setFilename(string file);
	/** Is the config object empty? For this to be a no, the directives object must
	    contain values for the keys conffilename and KeyboardType */
  	bool isEmpty();
	inline void setDirectives(const ConfigDirectives& directivelist) { directives = directivelist; }
	/** print */
	void print(ostream& out) const;
	/** clear the config object */
	void clear();
private:
	map<const string, LCommand>  comtable;
	ConfigDirectives directives;
	vector<string> vec;
};

#endif
