/***************************************************************************
                          lkbd.cpp  -  description
                             -------------------
    begin                : Fri Jan 17 2003
    copyright            : (C) 2003 by Sheldon Lee Wen
    email                : leewsb@hotmail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <lineak/lkbd.h>
#include <lineak/lconfig.h>
#include <lineak/lineak_util_functions.h>
#include <lineak/lkey.h>
#include <lineak/lcommand.h>

extern bool verbose;

int LKey::keycount=0;

LKbd::LKbd() {
	name = null;
	model = null;
	brand = null;
}
LKbd::LKbd(string iname, string ibrand, string imodel) {
	name = iname;
	brand = ibrand;
	model = imodel;
}
LKbd::~LKbd(){
}
const map<string, LKey>&  LKbd::getKeys() const {
	return keytable;
}
LKey& LKbd::getKey(string name) {
	return keytable[name];
}
LKey& LKbd::getKey(int keycode) {
    for (map<string, LKey>::iterator m = keytable.begin(); m != keytable.end(); m++) {
			if ( (m->second).keycode == keycode) {
				return (m->second);
			}
		}
}

vector<string> LKbd::getKeyNames() {
	vector<string> keynames; // = new Vector;

  for (map<string, LKey>::const_iterator m = keytable.begin(); m != keytable.end(); m++)
  	keynames.push_back(m->first);
	return keynames;
}

void LKbd::addKey(string name, int keycode) {
	keytable[name] = LKey(name,keycode, LCommand());
}
void LKbd::setCommands(LConfig &config) {
	/** Add commands to a togglable key */
	/** If it's a key with a play|pause form, for each of these forms, get the appropriate command */
  for (map<string, LKey>::iterator m = keytable.begin(); m != keytable.end(); m++) {
//     cout << (m->second) << endl << "command = " << endl;
//     (config.getCommand(m->first)).print(cout);
//     cout << endl;
    /** If it's a togglable key, and a command for both keys is not set. */
   	if ((m->second).isToggle() && (config.getCommand(m->first).isEmpty())) {
//    	if ((m->second).isToggle() && (config.getCommand(m->first) == null || config.getCommand(m->first) == "")) {
//     		cout << "Is a togglable key!\n";
     	vector<string> names = (m->second).getKeynames();
      for (vector<string>::const_iterator i = names.begin(); i != names.end(); i++) {
       	(m->second).setCommand(*i, config.getCommand(*i));
//          		(m->second).setCommand(*i, lineak_util_functions::strip_quotes(config.getCommand(*i)));
            	//cout << "adding command " <<  config.getCommand(*i) << " to keyname " << *i << endl;
      }
//            cout << (m->second) << endl;
   	}
    else {
    	/* If there the user is treating a togglable key as one key, turn it's toggle status off. But we leave the display toggle on. */
      if ((m->second).isToggle()) {
       	(m->second).setToggle(false);
//        	cout << "Turning off toggleable flag on key: " << (m->second).getKeyname() << endl;
      }
			(m->second).setCommand(config.getCommand(m->first));
//			(m->second).setCommand(lineak_util_functions::strip_quotes(config.getCommand(m->first)));
            //cout << "adding command " <<  config.getCommand(m->first) << " to keyname " << (m->first) << endl;
    }
	}
}
void LKbd::setCommand(string name, LCommand command) {
	keytable[name].setCommand(command);
}
/** Return a vector of all of the keycodes. */
vector<int> LKbd::getKeyCodes(){
	vector<int> keycodes;

    for (map<string, LKey>::const_iterator m = keytable.begin(); m != keytable.end(); m++)
        keycodes.push_back((m->second).keycode);

//     cout << "vector has " << keycodes.size() << " elements: { ";
//         for (vector<int>::const_iterator it = keycodes.begin(); it != keycodes.end(); it++)
//                 cout << *it << " ";
//         cout << "} " << endl;

    return keycodes;
	
}
/** Is the keyboard empty? */
bool LKbd::isEmpty() {
	// An empty keyboard is one that has no name. The rest can be empty data,
	// but we at least need a name.
	if (name == null)
		return true;
	return false;
}
void LKbd::print(ostream &out) const {
	// Don't print out empty keyboard entries.
	if (name != null) {
    	out << "[" << name << "]" << endl;
    	out << "     " << "brandname = " << brand << endl;
    	out << "     " << "modelname = " << model << endl;
     	out << "     " << "[KEYS]" << endl;
     	for (map< string, LKey>::const_iterator i = keytable.begin(); i != keytable.end(); i++)
      		out << "          " << i->first << "=" << (i->second).keycode << endl;
     	out << "     " << "[END KEYS]" << endl;
    	out << "[END " << name << "]" << endl;
    }
    else
    	cerr << "Attempting to output an empty keyboard!" << endl;
}

