/***************************************************************************
                          lkey.cpp  -  description
                             -------------------
    begin                : Fri Feb 28 2003
    copyright            : (C) 2003 by Sheldon Lee Wen
    email                : leewsb@hotmail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <lineak/lkey.h>

LKey::LKey(){
		keysym = 0x1008FF01 + keycount;
		keycount++;
		keyname = null;
		toggle = false;
		dtoggle = false;
}
LKey::~LKey(){
}
LKey::LKey(string name, int code, LCommand com) : keycode(code), keyname(name) {
		keysym = 0x1008FF01 + keycount;
		keycount++;
		commands[name] = com;
		toggle = false;
		dtoggle = false;
		init();
}
void LKey::init() {
    /** Handle keys with names of the form:
       		name1|name2|name3 */
    string key = keyname;
    int index;
    while (((unsigned int)(index = key.find('|'))) != string::npos) {
    		toggle = true;
		dtoggle = true;
        	string tmp = key.substr(0,index);
		keynames.push_back(tmp);
		display_names.push(tmp);
		key.erase(0, index+1);
     }
	keynames.push_back(key);
	display_names.push(key);
}

bool LKey::isToggle() const {
	return toggle;
}
void LKey::setKeyname(const string & name) {
    	keyname = name;
     	keynames.clear();
     	toggle = false;
        init();
}
void LKey::toggleKeyState() {
	if (toggle||dtoggle) {
		string name = display_names.front();
   		display_names.pop();
    		display_names.push(name);
  	}
}
/** Return the next name to be displayed. The display name is not rotated until
		you actually get the command with getCommand() This is to keep the display name
		and the command to be run in sync. */
string LKey::getDisplayName() {
	if (dtoggle) {
    		return display_names.front();
  	} else
     		return keyname;
}
/** If this is not a toggleable key, return the command that corresponds to the keyname
	as defined in the keyboard definitions file. Else, return the command for the current
	state of the key and put the current state at the end of the queue. */
LCommand LKey::getCommand() {
	if (!toggle) {
		return commands[keyname];
	}
	else {
 		return commands[display_names.front()];
	}
}
/** Set the command for a toggle state */
void LKey::setCommand(string name, LCommand command) {
	if (!command.isEmpty())
		commands[name] = command;
}
/** Set the command for the default keyname */
void LKey::setCommand(LCommand command) {
	commands[keyname] = command;
}

void LKey::print(ostream&out)  {
		if (keyname != null) {
			out << "key = " << keyname << endl;
			out << "keycode = " << keycode << endl;
			out << " keysym = " << keysym << endl;
				
			if (!isToggle()) {
				out << " toggle = false" << endl;
				out << " and command = " << commands[keyname] << endl;
			}
			else {
				out << " toggle = true" << endl;
				for (map<string,LCommand>::const_iterator i = commands.begin(); i != commands.end(); i++)
					out << "for displayname = " << i->first << " command = " << i->second << endl;
				out << " and command = " << commands[keyname] << endl;
			}
		}
    	else
    		cerr << "Attempting to output an empty key!" << endl;
}

ostream & operator<<(ostream &out, LKey &rhs) {
	rhs.print(out);
	return out;
}
