/**************************************************************************
                          xmgr.cpp  -  description
                             -------------------
    begin                : Tue Feb 18 2003
    copyright            : (C) 2003 by Sheldon Lee Wen
    email                : leewsb@hotmail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <lineak/xmgr.h>
#include <lineak/lkbd.h>
#include <lineak/lkey.h>
#include <signal.h>

#include <lineakd_core_functions.h>
#include <lineak/lineak_core_functions.h>

using namespace lineak_core_functions;
extern bool verbose;

bool Xmgr::caughtXerror = false;

Xmgr::Xmgr(string display) : displayname(display) {
  mask = 0L;

  screen = DefaultScreen (display.c_str());
  mask |= KeyPressMask | KeyReleaseMask;

  if (! (ourdisplay = XOpenDisplay (displayname.c_str())) ) {
        error("Could not open the display. You should exit!");
  }

  XSetErrorHandler(&Xhandler);
}

Xmgr::~Xmgr(){
}
/** Did we sucessfully initialize the display and xkbd? */
bool Xmgr::initialized() {
	if (!ourdisplay)
		return false;
	return true;
}
/* X error handler. When a BadValue error occurs during XkbSetMap, we want to retry */
/* Right now we are just ignoring errors */
int Xmgr::Xhandler(Display* theDisplay, XErrorEvent* xev) {
        if (( (int)(xev->request_code) ==150 || (int)(xev->request_code) ==149) && (int)(xev->minor_code) ==9) {
    	   caughtXerror = true;
           cerr << "*** Xlib error caught ***" << endl;
	   cerr << "Major opcode of failed request: " << (int)(xev->request_code) << " (XKEYBOARD)" << endl;
	   cerr << "Minor opcode of failed request: " << (int)(xev->minor_code) << " (XkbSetMap)" << endl;
	   cerr << "Resource ID of failed request: " << xev->resourceid << endl;
           cerr << "Serial number of failed request: " << xev->serial << endl;
           cerr << "Error code: " << (int)(xev->error_code) << endl;
           cerr << "Type: " << xev->type << endl;
	   cerr << "Maybe we should exit now?" << endl;
        }
  	return 0;
}
/* initialize X (xkb) so it understands the extra keys */
void Xmgr::cleanup(LKbd& myKbd) {
  if (ourdisplay) {

	vector<int> keycodes = myKbd.getKeyCodes();

  	/* add our new keysyms to the key codes of our display */
   	for (int i=0; i != (int)keycodes.size(); ) {
		int keycode = keycodes[i];

    	   /* clear the keysym from keycode */
           XUngrabKey(ourdisplay, keycode, AnyModifier, DefaultRootWindow(ourdisplay));
           i++;
  	}
  }
}
/* initialize X (xkb) so it understands the extra keys */
bool Xmgr::initialize(LKbd& myKbd) {
   if (ourdisplay) {
        XAllowEvents(ourdisplay, AsyncKeyboard, CurrentTime);
        XSelectInput(ourdisplay,DefaultRootWindow(ourdisplay),mask);

        vector<int> keycodes = myKbd.getKeyCodes();
	
  	/* add our new keysyms to the key codes of our display */

   	for (vector<int>::iterator it = keycodes.begin() ; it != keycodes.end();it++) {
                int keycode = *it;
                if (verbose) cout << "Mapping keycode: " << keycode << endl;
                XGrabKey(ourdisplay,keycode,AnyModifier,DefaultRootWindow(ourdisplay),0,GrabModeAsync,GrabModeAsync);
	}
   }
   else
        return false;

 return true;
}

